rm(list=ls())     # Borra todos los objetos que en ese momento estén en el ambiente.
gc()              # Vaciar papelera de RStudio.

# DESCRIPCION ####
#* Este sistema permite realizar escenarios de sensibilidad para evaluar el modelo de ajuste con un Fondo de Reserva (FR)
tiempo_0 <-Sys.time()

# Paqueterías ####
#********************************************************************************************************************
list.packages <-as.list(c("data.table" , "nlme"     , "stats",
                          "readstata13", "foreign"  , "openxlsx",
                          "stringr"    , "dplyr"    , "ggplot2",
                          'jrvFinance' , 'gridExtra'))

# se instalan los paquetes en caso de no esté instalado
invisible(lapply(list.packages, function(p) {if (!require(p, character.only = TRUE)) install.packages(p, dependencies = TRUE)}))


data.frame(paquete = unlist(list.packages),  instalado = unlist(lapply(list.packages, require, character.only = TRUE)))
remove(list.packages)

# Funciones ####
source("0_funciones_curvas_j.R", encoding = "UTF-8")

# Variabes a ingresar ####
#********************************************************************************************************************
nom_arch_resul <- paste0("Tablas_Resultados")
tiempo_anios <- 5

variables <- c("Inv", 
               "i_cred", "i_tes", 
               "esc_v")

# supuestos ####
anios_graf   <- tiempo_anios

Inv    <- seq(10000, 1000000, length.out=10)
MU     <- c(30, 50)/100

GF     <- c(30000)
GV     <- c(40000)
P_Max  <- c(300,3000)

i_inf  <- seq(0, 15,    length.out=10)/100
i_tes  <- seq(0, 20,    length.out=10)/100
i_cred <- seq(0, 100,   length.out=10)/100

esc_p <- 1:10
esc_v <- 1:10

esc_base <- data.frame(Inv          = Inv[4], 
                       GF           = GF[1], 
                       GV           = GV[1], 
                       P_Max        = P_Max[1], 
                       MU           = MU[1],
                       i_cred       = i_cred[5], 
                       i_inf        = i_inf[4],
                       i_tes        = i_tes[4], 
                       esc_p        = esc_p[7],
                       esc_v        = esc_v[6])

# Escenario base #####
{
  exist_arch <- file.exists(paste0(nom_arch_resul, ".xlsx"))
  
  if(exist_arch){
    resultados <- loadWorkbook(paste0(nom_arch_resul, ".xlsx"))
  } else{
    resultados <- createWorkbook()
  }
  
  ## Curva J ####
  escenarios <- list()
  escenarios[["base"]][["esc_0"]][["sin_ajuste"]] <- curva_j(bd = NULL, 
                                                             tiempo_anios = tiempo_anios,
                                                             GF = esc_base$GF, 
                                                             GV = esc_base$GV, 
                                                             P_Max = esc_base$P_Max, 
                                                             MU = esc_base$MU, 
                                                             Inv = esc_base$Inv, 
                                                             i_cred = esc_base$i_cred,
                                                             i_inf = esc_base$i_inf,
                                                             i_tes = esc_base$i_tes, 
                                                             esc_p = esc_base$esc_p, 
                                                             esc_v = esc_base$esc_v)
  # Curva J Ajustada ####
  escenarios[["base"]][["esc_0"]][["con_ajuste"]] <- ajuste_curva_j(bd = escenarios[["base"]][["esc_0"]][["sin_ajuste"]]$bd,
                                                                    Inv = esc_base$Inv, 
                                                                    i_cred = esc_base$i_cred,
                                                                    i_inf = esc_base$i_inf,
                                                                    i_tes = esc_base$i_tes)
  # graficas #####
  {
    bd_esc <- 
      full_join(escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd"]] %>% 
                  select(anio.t, t.t, B.t,        AB.t,        Inv_i_inv.t,        Inv_i_tes.t),
                escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd"]] %>% 
                  select(anio.t, t.t, B_ajuste.t, AB_ajuste.t, Inv_i_inv_ajuste.t, Inv_i_tes_ajuste.t),
                by = c("anio.t", "t.t")) %>% 
      filter(!is.na(B.t)) %>% 
      select(anio.t, t.t, B.t, B_ajuste.t, contains("AB"), contains("Inv_i_tes"), contains("Inv_i_inv"))
    
    ## B.t: con y sin ajuste  ####
    graf_B.t <-
      ggplot(bd_esc) +
      labs(x = "Año", y = "Unidades monetarias", title = "Flujos de efectivo de una empresa") +
      geom_vline(xintercept = 0, linetype = "solid", color = "gray40", size = 0.5) +
      geom_hline(yintercept = 0, linetype = "solid", color = "gray40", size = 0.5)+
      
      geom_line(mapping = aes(x = anio.t,
                              y = B.t,
                              color = "Beneficios")) +
      geom_point(mapping = aes(x = anio.t,
                               y = B.t,
                               color = "Beneficios"), 
                 size = 1.5) +
      
      geom_line(mapping = aes(x = anio.t,
                              y = B_ajuste.t,
                              color = "Beneficios ajustados")) +
      geom_point(mapping = aes(x = anio.t,
                               y = B_ajuste.t,
                               color = "Beneficios ajustados"),
                 size = 1) +
      
      scale_color_manual(values = c("Beneficios" = "#A2CD5A", 
                                    "Beneficios ajustados" = "#556B2F"))+
      scale_x_continuous(breaks=seq(0,anios_graf,0.5),
                         limits=c(0, anios_graf, 0.5))+
      scale_y_continuous(labels = scales::comma) +
      
      theme(
        #ajustar los margenes del gráfico
        plot.margin = margin(0.25, 1, 0.25, 0.1, "cm"), 
        #fondo del gráfico
        panel.background = element_rect(fill = "gray100", colour = "gray100", size = 2, linetype = "solid"), 
        #lineas del gráfico
        panel.grid.major = element_line(size = 0.25, linetype = 'solid', colour = "gray95"),    
        #líneas auxiliares
        panel.grid.minor = element_line(size = 0.5, linetype = 'solid', colour = "gray98"))
    
    ## AB.t con y sin ajuste #####
    graf_AB.t <-
      ggplot(bd_esc) +
      labs(x = "Año", y = "Unidades monetarias", title = "Acumulación de los flujos de efectivo de una empresa") +
      geom_vline(xintercept = 0, linetype = "solid", color = "gray40", size = 0.5) +
      geom_hline(yintercept = 0, linetype = "solid", color = "gray40", size = 0.5)+
      
      geom_line(mapping = aes(x = anio.t,
                              y = Inv_i_inv.t,
                              color = "Inv_i_inv.t"),
                linetype = "dashed") +
      geom_line(mapping = aes(x = anio.t,
                              y = Inv_i_inv_ajuste.t,
                              color = "Inv_i_inv_ajuste.t"),
                linetype = "dashed") +
      geom_line(mapping = aes(x = anio.t,
                              y = AB.t,
                              color = "AB.t")) +
      
      geom_line(mapping = aes(x = anio.t,
                              y = Inv_i_tes.t,
                              color = "Inv_i_tes.t"),
                linetype = "13")  +
      geom_line(mapping = aes(x = anio.t,
                              y = Inv_i_tes_ajuste.t,
                              color = "Inv_i_tes_ajuste.t"),
                linetype = "13") +
      
      geom_line(mapping = aes(x = anio.t,
                              y = AB_ajuste.t,
                              color = "AB_ajuste.t")) +
      scale_color_manual(name = "Etiquetas",
                         values = c("Inv_i_inv.t"        = "#A0F0C0", 
                                    "Inv_i_inv_ajuste.t" = "#00BE4B", 
                                    "AB.t"               = "#BCBAFF", 
                                    "AB_ajuste.t"        = "#0028BB", 
                                    "Inv_i_tes.t"        = "gray60", 
                                    "Inv_i_tes_ajuste.t" = "gray10"))+
      
      scale_x_continuous(breaks=seq(0,anios_graf, 0.5),
                         limits=  c(0,anios_graf, 0.5))+
      scale_y_continuous(labels = scales::comma) +
      
      # Aspectos estétivos del gráfico
      theme(
        #ajustar los margenes del gráfico
        plot.margin = margin(0.25, 1, 0.25, 0.1, "cm"), 
        #fondo del gráfico
        panel.background = element_rect(fill = "gray100", colour = "gray100", size = 2, linetype = "solid"), 
        #lineas del gráfico
        panel.grid.major = element_line(size = 0.25, linetype = 'solid', colour = "gray95"),    
        #líneas auxiliares
        panel.grid.minor = element_line(size = 0.5, linetype = 'solid', colour = "gray98"))
    
    # save graficos #####
    escenarios[["base"]][[paste0("esc_", 0)]][["graf_B.t"]] <- graf_B.t
    escenarios[["base"]][[paste0("esc_", 0)]][["graf_AB.t"]] <- graf_AB.t
  }
  
  ## save datos ####
  insumos <- data.frame(variable = names(escenarios$base$esc_0$sin_ajuste$bd_insumos), 
                        valor = t(escenarios$base$esc_0$sin_ajuste$bd_insumos)) 
  rownames(insumos) <- NULL
  
  escenario_base <- data.frame(t(full_join(escenarios$base$esc_0$sin_ajuste$bd_res, 
                                           escenarios$base$esc_0$con_ajuste$bd_res)[, -1]))
  names(escenario_base) <- c("Sin_Ajuste", "Con_Ajuste")
  escenario_base <- data.frame(variables = rownames(escenario_base),
                               Sin_Ajuste = escenario_base$Sin_Ajuste,
                               Con_Ajuste = escenario_base$Con_Ajuste)
  
  
  if(exist_arch==F){ addWorksheet(resultados, "base")}
  writeData(resultados, 
            sheet = "base", 
            x = insumos, 
            startCol = 1, 
            startRow = 2)  
  
  writeData(resultados, 
            sheet = "base", 
            x = escenario_base, 
            startCol = ncol(insumos) + 2, 
            startRow = 2)
  
  if(exist_arch==F){ addWorksheet(resultados, "base datos")}
  writeData(resultados, 
            sheet = "base datos", 
            x = bd_esc, 
            startCol = 1, 
            startRow = 1) 
}

# Escenarios Sensibilidad #####
comp <- list()
for(var in variables){
  esc_base_aux <- esc_base
  
  var_new= get(var)
  n= length(var_new)
  for (i in 1:length(var_new)) {
    
    esc_base_aux[[var]] = var_new[i]
    
    ## Curva J  #####
    escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]] <- 
      curva_j(bd = NULL, 
              tiempo_anios = tiempo_anios,
              GF           = esc_base_aux$GF, 
              GV           = esc_base_aux$GV, 
              P_Max        = esc_base_aux$P_Max, 
              MU           = esc_base_aux$MU, 
              Inv          = esc_base_aux$Inv, 
              i_cred       = esc_base_aux$i_cred,
              i_inf        = esc_base_aux$i_inf,
              i_tes        = esc_base_aux$i_tes, 
              esc_p        = esc_base_aux$esc_p, 
              esc_v        = esc_base_aux$esc_v)
    
    # Curva J Ajustada #####
    escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]] <- 
      ajuste_curva_j(bd     =  escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd"]],
                     Inv    = esc_base_aux$Inv, 
                     i_cred = esc_base_aux$i_cred,
                     i_inf  = esc_base_aux$i_inf,
                     i_tes  = esc_base_aux$i_tes)
    
    # Comparativo #####
    (comp_aux  <-
       data.frame(variable         = var,
                  escenario        = i, 
                  valor_var        = esc_base_aux[[var]],
                  Inv              = escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_res"]]$Inv,
                  Fondo_reserva    = escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$FR,
                  Inv_ajuste       = escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$Inv,
                  # amplia_Inv       = escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$Inv/escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_res"]]$Inv,
                  n_tes            = escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_res"]]$n_tes,
                  n_tes_ajuste     = escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$n_tes,
                  # n_tes_dif        = (escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$n_tes - 
                  #                       escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_res"]]$n_tes),
                  # i_tes            = escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_insumos"]]$i_tes*100,
                  i_inv            = escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_res"]]$i_inv,
                  i_inv_ajuste     = escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$i_inv,
                  # i_inv_dif_pb     = 100*(escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd_res"]]$i_inv-
                  #                           escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd_res"]]$i_inv),
                  AB.n             = escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd"]]$AB.t[length(escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd"]]$AB.t)],
                  AB_ajuste.n      = escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd"]]$AB_ajuste.t[length(escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd"]]$AB_ajuste.t)])
     # mutate(amplia_AB.n = AB.n/AB_ajuste.n)
    )
    if (i==1){
      (comparativo  <- 
         data.frame(variable         = "base",
                    escenario        = 0, 
                    valor_var        = esc_base[[var]],
                    Inv              = escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_res"]]$Inv,
                    Fondo_reserva    = escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$FR,
                    Inv_ajuste       = escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$Inv,
                    # amplia_Inv       = escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$Inv/escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_res"]]$Inv,
                    n_tes            = escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_res"]]$n_tes,
                    n_tes_ajuste     = escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$n_tes,
                    # n_tes_dif        = (escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$n_tes - 
                    #                       escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_res"]]$n_tes),
                    # i_tes            = escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_insumos"]]$i_tes*100,
                    i_inv            = escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_res"]]$i_inv,
                    i_inv_ajuste     = escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$i_inv,
                    # i_inv_dif_pb     = 100*(escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd_res"]]$i_inv - 
                    #                           escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd_res"]]$i_inv),
                    AB.n             = escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd"]]$AB.t[length(escenarios[["base"]][[paste0("esc_", 0)]][["sin_ajuste"]][["bd"]]$AB.t)],
                    AB_ajuste.n      = escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd"]]$AB_ajuste.t[length(escenarios[["base"]][[paste0("esc_", 0)]][["con_ajuste"]][["bd"]]$AB_ajuste.t)])
       # mutate(amplia_AB.n = AB.n/AB_ajuste.n)
      )
      
      comp[[var]] <- rbind(comparativo, comp_aux)
    }else{
      comp[[var]] <- rbind(comp[[var]], comp_aux)
    }
    # graficas #####
    {
      bd_esc <- 
        full_join(escenarios[[var]][[paste0("esc_", i)]][["sin_ajuste"]][["bd"]] %>% 
                    select(anio.t, t.t, B.t,        AB.t,        Inv_i_inv.t,        Inv_i_tes.t),
                  escenarios[[var]][[paste0("esc_", i)]][["con_ajuste"]][["bd"]] %>% 
                    select(anio.t, t.t, B_ajuste.t, AB_ajuste.t, Inv_i_inv_ajuste.t, Inv_i_tes_ajuste.t),
                  by = c("anio.t", "t.t")) %>% 
        filter(!is.na(B.t))
      
      ## B.t: con y sin ajuste  ####
      graf_B.t <-
        ggplot(bd_esc) +
        labs(x = "Año", y = "Unidades monetarias", title = "Flujos de efectivo de una empresa") +
        geom_vline(xintercept = 0, linetype = "solid", color = "gray40", size = 0.5) +
        geom_hline(yintercept = 0, linetype = "solid", color = "gray40", size = 0.5)+
        
        geom_line(mapping = aes(x = anio.t,
                                y = B.t,
                                color = "Beneficios")) +        
        geom_point(mapping = aes(x = anio.t,
                                 y = B.t,
                                 color = "Beneficios"), 
                   size = 1.5) +
        
        geom_line(mapping = aes(x = anio.t,
                                y = B_ajuste.t,
                                color = "Beneficios ajustados")) +
        geom_point(mapping = aes(x = anio.t,
                                 y = B_ajuste.t,
                                 color = "Beneficios ajustados"), 
                   size = 1) +
        
        scale_color_manual(values = c("Beneficios" = "#A2CD5A", 
                                      "Beneficios ajustados" = "#556B2F"))+
        scale_x_continuous(breaks=seq(0,anios_graf,0.5),
                           limits=c(0, anios_graf, 0.5))+
        scale_y_continuous(labels = scales::comma) +
        
        
        theme(
          #ajustar los margenes del gráfico
          plot.margin = margin(0.25, 1, 0.25, 0.1, "cm"), 
          #fondo del gráfico
          panel.background = element_rect(fill = "gray100", colour = "gray100", size = 2, linetype = "solid"), 
          #lineas del gráfico
          panel.grid.major = element_line(size = 0.25, linetype = 'solid', colour = "gray95"),    
          #líneas auxiliares
          panel.grid.minor = element_line(size = 0.5, linetype = 'solid', colour = "gray98"))
      
      # AB.t con y sin ajuste #####
      graf_AB.t <-
        ggplot(bd_esc) +
        labs(x = "Año", y = "Unidades monetarias", title = "Acumulación de los flujos de efectivo de una empresa") +
        geom_vline(xintercept = 0, linetype = "solid", color = "gray40", size = 0.5) +
        geom_hline(yintercept = 0, linetype = "solid", color = "gray40", size = 0.5)+
        
        geom_line(mapping = aes(x = anio.t,
                                y = Inv_i_inv.t,
                                color = "Inv_i_inv.t"),
                  linetype = "73", 
                  size=1) +
        geom_line(mapping = aes(x = anio.t,
                                y = Inv_i_inv_ajuste.t,
                                color = "Inv_i_inv_ajuste.t"),
                  linetype = "73", 
                  size= 1) +
        
        geom_line(mapping = aes(x = anio.t,
                                y = AB.t,
                                color = "AB.t"),
                  linetype = "solid", 
                  size=1) +
        geom_line(mapping = aes(x = anio.t,
                                y = AB_ajuste.t,
                                color = "AB_ajuste.t"),
                  linetype = "solid", 
                  size=1) +
        
        geom_line(mapping = aes(x = anio.t,
                                y = Inv_i_tes.t,
                                color = "Inv_i_tes.t"),
                  linetype = "13")  +
        geom_line(mapping = aes(x = anio.t,
                                y = Inv_i_tes_ajuste.t,
                                color = "Inv_i_tes_ajuste.t"),
                  linetype = "13") +
        
        
        scale_color_manual(name = "Etiquetas",
                           values = c("Inv_i_inv.t"        = "#A0F0C0", 
                                      "Inv_i_inv_ajuste.t" = "#00BE4B", 
                                      "AB.t"               = "#BCBAFF", 
                                      "AB_ajuste.t"        = "#0028BB", 
                                      "Inv_i_tes.t"        = "gray60", 
                                      "Inv_i_tes_ajuste.t" = "gray10"))+
        
        scale_x_continuous(breaks=seq(0,anios_graf, 0.5),
                           limits=  c(0,anios_graf))+
        scale_y_continuous(labels = scales::comma) +
        
        
        # Aspectos estétivos del gráfico
        theme(
          #ajustar los margenes del gráfico
          plot.margin = margin(0.25, 1, 0.25, 0.1, "cm"), 
          #fondo del gráfico
          panel.background = element_rect(fill = "gray100", colour = "gray100", size = 2, linetype = "solid"), 
          #lineas del gráfico
          panel.grid.major = element_line(size = 0.25, linetype = 'solid', colour = "gray95"),    
          #líneas auxiliares
          panel.grid.minor = element_line(size = 0.5, linetype = 'solid', colour = "gray98"))
      
      # save graficos #####
      escenarios[[var]][[paste0("esc_", i)]][["graf_B.t"]] <- graf_B.t
      escenarios[[var]][[paste0("esc_", i)]][["graf_AB.t"]] <- graf_AB.t
    }
  }
  ## save comparativo ####
  escenarios[[var]][["comparativo"]] <- comp[[var]][order(comp[[var]]$valor_var),]
  
  
  if(exist_arch==F){addWorksheet(resultados, var)}
  writeData(resultados, 
            sheet = var, 
            x = escenarios[[var]][["comparativo"]], 
            startCol = 1, 
            startRow = 2)
}
saveWorkbook(resultados, paste0(nom_arch_resul, ".xlsx"), overwrite = T)

save(escenarios,file =  paste0(nom_arch_resul, ".RData"))

# Limpieaza ####
rm(list = setdiff(ls(), c("escenarios", "variables")))


# base ####
i <- 0
var <- "base"
{
  graf_B.t      <- escenarios[[var]][[paste0("esc_", i)]][["graf_B.t"]]
  graf_AB.t     <- escenarios[[var]][[paste0("esc_", i)]][["graf_AB.t"]]
  graf_AB_Inv.t <- escenarios[[var]][[paste0("esc_", i)]][["graf_AB_Inv.t"]]
  
  layout <- rbind(c(1),
                  c(2))
  
  # Crear una vista de cuadrícula que contenga los gráficos y la tabla
  grid.arrange(graf_B.t, graf_AB.t, layout_matrix = layout)
  print(escenarios$base$esc_0$sin_ajuste$bd_insumos)
  full_join(escenarios$base$esc_0$sin_ajuste$bd_res, 
            escenarios$base$esc_0$con_ajuste$bd_res)
}

