# Escenario #####
esc_B.t <- function(esc_v){
  
  bd_esc <- data.frame(esc = c(1:10),
                       alpha_0 = seq(0.3, 0.7, length.out=10)[order(seq(0, 1, length.out=10), decreasing = T)],
                       alpha_p = seq(0.5, 1, length.out=10),
                       n_pee   = seq(0.2, 1.5, length.out=10)[order(seq(0, 1, length.out=10), decreasing = T)],
                       n       = seq(1.5, 3, length.out=10)[order(seq(0, 1, length.out=10), decreasing = T)])    
  bd_esc  <- bd_esc[bd_esc$esc == esc_v, ]

  return(bd_esc)
}

# Función de producción ####
produccion <- function(escenario, p_max, PE_P_Max){
  vol_produccion <- ceiling(PE_P_Max+escenario*((p_max-PE_P_Max)/10))
  return(vol_produccion)
}

# Funcion ventas ####
ventas <- function(anio, 
                   PV, GF, GVE, P, 
                   variables_ventas){
  alpha_0    = variables_ventas$alpha_0
  alpha_p    = variables_ventas$alpha_p 
  n_pee      = variables_ventas$n_pee
  n          = variables_ventas$n
  
  
  B_0 <- -(GF + GVE*P )
  B_p <- PV*P - (GF + GVE*P )

  reescalador <- function(x,a,b,c,d){return(((x - a) / (b - a)) * (d - c) + c)}

  Beneficios<- c(reescalador(x =pnorm(-20:-3,  mean = 0, sd = 5, lower.tail = T), 
                             a = 0, b = pnorm(-3, mean = 0, sd = 5, lower.tail = T), 
                             c = alpha_0*B_0, d = 0),
                 reescalador(x = pnorm(-2:20, mean = 0, sd = 5, lower.tail = T), 
                             a = pnorm(-3, mean = 0, sd = 5, lower.tail = T), b = 1, 
                             c = 0, d = alpha_p*B_p))
  
  tiempo <- c(reescalador(x = -20:-3, 
                          a = -20, b = -3, 
                          c = 0, d = n_pee),
              reescalador(x = -2:20, 
                          a = -2.5, b = 20, 
                          c = n_pee, d = n))
  
  bd_B.t <- data.frame(tiempo, Beneficios)
  graf_B.t <- 
    ggplot(data = bd_B.t, 
           mapping = aes(x= tiempo, 
                         y=Beneficios))+
    geom_point()

  beneficio <- max(bd_B.t$Beneficios[anio>=tiempo])
  
  vol_ventas <- (beneficio +(-B_0))/PV
  
  vol_ventas <- ceiling(vol_ventas)
  return(list(vol_ventas = vol_ventas,
              bd_B.t     = bd_B.t, 
              graf_B.t   = graf_B.t ))
}

# funcion J #####
curva_j <- function(bd, 
                    tiempo_anios,
                    GF, GV,   P_Max, MU,
                    PV,    Inv,
                    i_cred, i_inf, i_tes,
                    esc_p,
                    esc_v){
  # Escenario ####
  variables_ventas <- esc_B.t(esc_v = esc_v)
  
  # Tasas mensuales ####
  i_cred.12 <- ((1+i_cred)^(1/12))-1
  i_tes.12  <- ((1+i_tes)^(1/12))-1
  i_inf.12  <- ((1+i_inf)^(1/12))-1
  
  # variables de produccion ####
  GFE <- GF/P_Max
  GVE <- GV/P_Max
  PV = (GFE + GVE)/(1-MU)
  
  P_Min  <- ceiling(GF/(PV-GVE))
  B_PMax <- ((PV*P_Max)-(GF+GVE*P_Max))
  
  PE_P_Max     <- (GF+GVE*P_Max)/PV
  
  # Validaciones de la base de datos ####
  if(is.null(bd)){
    anio.t    <- 0 
    t.t       <- 0
    Inv.t     <- Inv
    
    (bd <- data.frame(anio.t,t.t, 
                      Inv.t))
  }
  bd <- as.data.table(bd)
  
  
  variables_exist <- c("P.t", "PORC_PE_P.t", "PE_P.t", "V.t", "G.t", "I.t")
  
  variables_na <- c("B.t", "B_P_P.t", "Int_AB.t", "AB.t")
  
  
  for(v in variables_exist){
    if((v %in% names(bd))==F){
      bd[, c(v):= NA]
    }
  }
  for(v in variables_na){
    bd[, c(v):= NA]
  }
  
  bd <- as.data.frame(select(bd, 
                             anio.t,      t.t, 
                             all_of(variables_exist),
                             B.t, B_P_P.t, Int_AB.t, AB.t))
  
  
  # Contrucción de la curva J ####
  n_exp <- bd$t.t[length(bd$t.t)]
  for(t in 1:(12*tiempo_anios)){ # tiempo sea menor o igual a 10 años
    
    # Indicador de posición en el vector para el "momento" t
    i_t= t+1    
    ## Nuevo registro ####
    bd[i_t, ] <- NA
    
    # Periodo
    bd$anio.t[i_t] <- t/12
    bd$t.t[i_t] <- t
    
    # Produccion  ####
    bd$P.t[i_t] <- produccion(escenario = esc_p, 
                              p_max     = P_Max, 
                              PE_P_Max        = PE_P_Max)
    
    # Ventas ####
    V.t <- ventas( anio       = bd$anio.t[i_t],
                   P          = bd$P.t[i_t], 
                   PV         = PV, 
                   GF         = GF, 
                   GVE        = GVE,
                   variables_ventas = variables_ventas)
    bd$V.t[i_t] <- V.t$vol_ventas
    
    # Beneficios B.t #####
    # PE(P): Mínimo de ventas para tener ganancias
    bd$PE_P.t[i_t]      = ceiling((GF+GVE*bd$P.t[i_t])/PV)
    bd$PORC_PE_P.t[i_t] = 100*bd$PE_P.t[i_t]/bd$P.t[i_t]
    
    # Gastos
    bd$G.t[i_t] <- (GF + GVE*bd$P.t[i_t])*((1+i_inf.12)^bd$t.t[i_t])
    
    # Ingresos
    bd$I.t[i_t] <- (PV*bd$V.t[i_t])*((1+i_inf.12)^bd$t.t[i_t])
    
    
    #Funciones del Beneficio
    bd$B.t[i_t]     <-       bd$I.t[i_t]                             - bd$G.t[i_t]
    bd$B_P_P.t[i_t] <- bd$P.t[i_t]*((PV)*((1+i_inf.12)^bd$t.t[i_t])) - bd$G.t[i_t]
    
    # Acumulación del beneficio ####
    if(t==1){
      bd$AB.t[i_t]=bd$B.t[i_t] 
    }else{ 
      # si AB.t<0 se aplica tasa de prestamos, si no, se reinvierte en CETES
      int.t <- ifelse(bd$AB.t[i_t-1]<0, i_cred.12, i_tes.12) 
      bd$Int_AB.t[i_t] <- bd$AB.t[i_t-1]*(int.t)
      bd$AB.t[i_t] <- bd$AB.t[i_t-1]+  bd$Int_AB.t[i_t]  + bd$B.t[i_t]
    }
  }
  
  
  # Resultados #####
  {
    # i_inv ####
    # tasa de rentabilidad en el tiempo T con riesgos del mercado.
    i_inv.12 = ifelse(bd$AB.t[length(bd$t.t)]>0,
                      ((bd$AB.t[length(bd$t.t)]/Inv)^(1/max(bd$t.t)))-1,
                      NA)
    if(is.na(i_inv.12)){
      i_inv.12 <- NA
      i_inv    <- NA    
    }else{
      i_inv <- (((1+i_inv.12)^12) -1)
    }
    
    bd <- 
      bd %>% 
      mutate(Inv_i_inf.t   = Inv*(1+i_inf.12)^t.t, 
             Inv_i_tes.t   = Inv*(1+i_tes.12)^t.t, 
             i_inv.12      = i_inv.12,
             Inv_i_inv.t   = ifelse(is.na(i_inv.12), NA, Inv*(1+i_inv.12)^t.t), 
             i_inv.12      = NULL,
             n_0           = t.t*ifelse(Inv < AB.t, 1, 0), 
             n_inf         = t.t*ifelse(Inv_i_inf.t < AB.t, 1, 0), 
             n_tes         = t.t*ifelse(Inv_i_tes.t < AB.t, 1, 0))
    
    # n_0 ####
    # es el momento en que se paga la deuda si la i_inv = i_inf
    n_0 <- ifelse(sum(bd$n_0[bd$t.t!=0])==0, 
                  NA, 
                  min(bd$n_0[bd$t.t!=0 & bd$n_0!=0 ], na.rm = T))
    n_0 <- ifelse(n_0 == 0, NA, n_0)/12
    
    # n_inf ####
    # es el momento en que se paga la deuda si la i_inv = i_inf
    n_inf <- ifelse(sum(bd$n_inf[bd$t.t!=0])==0, 
                    NA, 
                    min(bd$n_inf[bd$t.t!=0 & bd$n_inf!=0 ], na.rm = T))
    n_inf <- ifelse(n_inf == 0, NA, n_inf)/12
    
    
    # n_tes ####
    # es el momento en que se paga la deuda si la i_inv = i_tes
    n_tes <- ifelse(sum(bd$n_tes[bd$t.t!=0])==0, 
                    NA, 
                    min(bd$n_tes[bd$t.t!=0 & bd$n_tes!=0 ], na.rm = T))
    n_tes <- ifelse(n_tes == 0, NA, n_tes)/12
    
    
    # tabla de resultados ####
    
    bd_res <-   data.frame(ajuste = "Sin_ajuste",
                           Inv,
                           n_0,
                           n_inf,
                           n_tes,
                           i_inv)
    bd_insumos <- data.frame(GF,
                             GV, 
                             GFE,
                             GVE,
                             GTE  = GFE+GVE,
                             PV, 
                             MU, 
                             i_cred,
                             i_inf,
                             i_tes,
                             esc_prod = esc_p,
                             esc_vent = esc_v,
                             P_Min,
                             PE_P = bd$PE_P.t[2] , 
                             Vol_PE_P = bd$PE_P.t[2]/bd$P.t[2],
                             Prod = bd$P.t[2],
                             Vol_PE_P_Max = PE_P_Max/P_Max,
                             P_Max, 
                             B.0_P = -(GF +GVE* bd$P.t[2]),
                             B.P_P = bd$B_P_P.t[2])
    
    return(list( bd         = bd, 
                 bd_insumos = bd_insumos, 
                 bd_res     = bd_res,
                 bd_B.t     = V.t$bd_B.t,
                 graf_B.t   = V.t$graf_B.t,
                 puntos_B.t = variables_ventas))
  }
}

# Ajuste funcion J #####
ajuste_curva_j <- function(bd, Inv,
                           i_cred, i_inf, i_tes){
  # Tasas mensuales ####
  i_cred.12 <- ((1+i_cred)^(1/12))-1
  i_tes.12  <- ((1+i_tes)^(1/12))-1
  i_inf.12  <- ((1+i_inf)^(1/12))-1
  
  # Fondo de reserva ####
  bd <- 
    bd %>% 
    mutate(FR.k = ifelse(!is.na(B.t) & B.t<0, (-B.t)*(1+i_tes.12)^(-t.t),0),
           FR = ifelse(t.t==0, sum(FR.k), NA ),
           B_ajuste.t      = NA,
           Int_AB_ajuste.t = NA,
           AB_ajuste.t     = NA) %>% 
    
    select(anio.t, t.t, 
           B.t, B_P_P.t, 
           FR.k, FR, 
           B_ajuste.t,
           Int_AB_ajuste.t, AB_ajuste.t)
  
  Inv_ajuste <- Inv+ bd$FR[1]
  
  # Ajuste de Curva J ####
  
  n_exp <- bd$t.t[length(bd$t.t)]
  for(t in 1:(length(bd[,1])-1)){
    
    # Indicador de posición en el vector para el "momento" t
    i_t= t+1
    
    ## Final del periodo ####
    bd$FR[i_t] <- bd$FR[i_t-1]*(1+i_tes.12)
    
    # Ajuste del beneficio B.t ####
    if(bd$B.t[i_t]<0 & bd$FR[i_t]>0){ # Se ajustan beneficios negativos si hay un fondo
      if(round(bd$FR[i_t], 3)>=round(abs(bd$B.t[i_t]),3)){  # Se ajusta si hay un fondo suficiente.
        FR.k <- abs(bd$B.t[i_t])
        bd$B_ajuste.t[i_t] <- bd$B.t[i_t]+FR.k
        bd$FR[i_t] <- bd$FR[i_t-1]*(1+i_tes.12) - FR.k
        
        # AB_ajuste.t: Acumulación del beneficio
        if(t==1){
          bd$AB_ajuste.t[i_t]=bd$B_ajuste.t[i_t]
        }else{
          int.t            <- ifelse(bd$AB_ajuste.t[i_t-1]<0, i_cred.12, i_tes.12)
          
          bd$Int_AB_ajuste.t[i_t] <- bd$AB_ajuste.t[i_t-1]*(int.t)
          bd$AB_ajuste.t[i_t]     <- bd$AB_ajuste.t[i_t-1]+  bd$Int_AB_ajuste.t[i_t]  + bd$B_ajuste.t[i_t]
        }
        
      }else{ # No hay fondo suficiente para hacer el ajuste
        bd$B_ajuste.t[i_t] <- bd$B.t[i_t]
        FR.k <- bd$FR[i_t]
        bd$FR[i_t] <- 0
        
        # Acumulación del beneficio
        if(t==1){
          bd$AB_ajuste.t[i_t] <- bd$B_ajuste.t[i_t]
        }else{
          int.t            <- ifelse(bd$AB_ajuste.t[i_t-1]<0, i_cred.12, i_tes.12)
          
          bd$Int_AB_ajuste.t[i_t] <- bd$AB_ajuste.t[i_t-1]*(int.t)
          
          # Se suma el fondo remanente FR.k
          bd$AB_ajuste.t[i_t]     <- bd$AB_ajuste.t[i_t-1] +  bd$Int_AB_ajuste.t[i_t]  + bd$B_ajuste.t[i_t] + FR.k 
        }
      }
      
    }else{
      bd$B_ajuste.t[i_t] <- bd$B.t[i_t]
      
      # Acumulación del beneficio
      if(t==1){
        bd$AB_ajuste.t[i_t]=bd$B_ajuste.t[i_t]
      }else{
        int.t <- ifelse(bd$AB_ajuste.t[i_t-1]<0, i_cred.12, i_tes.12)
        bd$Int_AB_ajuste.t[i_t] <- bd$AB_ajuste.t[i_t-1]*(int.t)
        bd$AB_ajuste.t[i_t]     <- bd$AB_ajuste.t[i_t-1]+  bd$Int_AB_ajuste.t[i_t]  + bd$B_ajuste.t[i_t]
      }
    }
  }
  
  # Resultados #####
  
  ## i_inv ####
  # tasa de rentabilidad en el tiempo T con riesgos del mercado.
  i_inv_ajuste.12 = ifelse(bd$AB_ajuste.t[length(bd$t.t)]>0,
                           ((bd$AB_ajuste.t[length(bd$t.t)]/Inv_ajuste)^(1/max(bd$t.t)))-1,
                           NA)
  if(is.na(i_inv_ajuste.12)){
    i_inv_ajuste.12 <- NA
    i_inv_ajuste    <- NA
  }else{
    i_inv_ajuste <- (((1+i_inv_ajuste.12)^12)-1)
  }
  
  bd <- 
    bd %>% 
    mutate(Inv_i_inf_ajuste.t   = Inv_ajuste*(1+i_inf.12)^t.t, 
           Inv_i_tes_ajuste.t   = Inv_ajuste*(1+i_tes.12)^t.t, 
           i_inv_ajuste.12      = i_inv_ajuste.12,
           Inv_i_inv_ajuste.t   = ifelse(is.na(i_inv_ajuste.12), NA, Inv_ajuste*(1+i_inv_ajuste.12)^t.t), 
           i_inv_ajuste.12      = NULL,
           n_0_ajuste           = t.t*ifelse(Inv_ajuste < AB_ajuste.t, 1, 0), 
           n_inf_ajuste         = t.t*ifelse(Inv_i_inf_ajuste.t < AB_ajuste.t, 1, 0), 
           n_tes_ajuste         = t.t*ifelse(Inv_i_tes_ajuste.t < AB_ajuste.t, 1, 0))
  
  ## n_0 ####
  # es el momento en que se paga la deuda si la i_inv = i_inf
  n_0_ajuste <- ifelse(sum(bd$n_0_ajuste[bd$t.t!=0])==0, 
                NA, 
                min(bd$n_0_ajuste[bd$t.t!=0 & bd$n_0_ajuste!=0 ], na.rm = T))
  n_0_ajuste <- ifelse(n_0_ajuste == 0, NA, n_0_ajuste)/12
  
  ## n_inf ####
  # es el momento en que se paga la deuda si la i_inv = i_inf
  n_inf_ajuste <- ifelse(sum(bd$n_inf_ajuste[bd$t.t!=0])==0, 
                  NA, 
                  min(bd$n_inf_ajuste[bd$t.t!=0 & bd$n_inf_ajuste!=0 ], na.rm = T))
  n_inf_ajuste <- ifelse(n_inf_ajuste == 0, NA, n_inf_ajuste)/12
  
  
  ## n_tes ####
  # es el momento en que se paga la deuda si la i_inv = i_tes
  n_tes_ajuste <- ifelse(sum(bd$n_tes_ajuste[bd$t.t!=0])==0, 
                  NA, 
                  min(bd$n_tes_ajuste[bd$t.t!=0 & bd$n_tes_ajuste!=0 ], na.rm = T))
  n_tes_ajuste <- ifelse(n_tes_ajuste == 0, NA, n_tes_ajuste)/12
  
  ## tabla de resultados #####
  bd_res <-   data.frame(ajuste    = "Con ajuste",
                         FR        = bd$FR[1],
                         Inv       = Inv_ajuste,
                         n_0       = n_0_ajuste,
                         n_inf     = n_inf_ajuste,
                         n_tes     = n_tes_ajuste,
                         i_inv     = i_inv_ajuste )
  
  
  return(list(bd=bd, bd_res= bd_res))
}








